@@ -55,6 +55,7 @@ class Dfs2(_Dfs123):
             raise Exception(f"file {self._filename} does not exist!")
 
         self._dfs = DfsFileFactory.Dfs2FileOpen(self._filename)
+        self._source = self._dfs
         self._dx = self._dfs.SpatialAxis.Dx
         self._dy = self._dfs.SpatialAxis.Dy
         self._nx = self._dfs.SpatialAxis.XCount
@@ -69,7 +70,9 @@ class Dfs2(_Dfs123):
         return self.find_nearest_elements(lon, lat)
 
     def find_nearest_elements(
-        self, lon, lat,
+        self,
+        lon,
+        lat,
     ):
         """Find index of closest element
 
@@ -84,7 +87,7 @@ class Dfs2(_Dfs123):
         Returns
         -------
 
-        (int,int): indexes in y, x 
+        (int,int): indexes in y, x
         """
         projection = self._dfs.FileInfo.Projection
         axis = self._dfs.SpatialAxis
@@ -177,18 +180,15 @@ class Dfs2(_Dfs123):
                 self._dy,
             )
         )
-    
 
     @property
     def dx(self):
-        """Step size in x direction
-        """
+        """Step size in x direction"""
         return self._dx
 
     @property
     def dy(self):
-        """Step size in y direction
-        """
+        """Step size in y direction"""
         return self._dy
 
     @property
@@ -222,9 +222,9 @@ class Dfs2(_Dfs123):
             dy: float
                 length of each grid in the y direction (projection units)
             latitude_origin: float, optional
-                latitude at origin of new grid, default same as original 
+                latitude at origin of new grid, default same as original
             longitude_origin: float, optional
-                longitude at origin of new grid, default same as original 
+                longitude at origin of new grid, default same as original
             nx: int, optional
                 n grid points in x direction, default same as original
             ny: int, optional
